#サポートツール
require 'rubygems'
require 'active_support/all'
require 'rest_client'
require 'yaml'
require 'json'
require 'base64'

if ARGV.size < 1
  puts "uploader.rb host dirname license_name email token"
  puts "upload on child dirs"
  exit
end

host = ARGV[0]
dirname = ARGV[1]
lsname = ARGV[2]
email = ARGV[3]
auth = ARGV[4]

class Uploader
  def initialize host, dirname, lsname, email, auth
    @host = host
    @dirname = dirname
    @lsname = lsname
    @email = email
    @auth = auth
    @licenses = self.licenses
    @license_groups = self.license_groups
  end
  
  def licenses
    res = {}
    u = @host + 'licenses.json'
    r = RestClient.get(u, {email: @email, token: @auth})
    ary = JSON.parse r
    ary['list'].each do |l|
      res[l['name']] = l
    end
    res
  end
  
  def license_groups
    res = {}
    u = @host + 'license_groups.json'
    r = RestClient.get(u, {email: @email, token: @auth})
    ary = JSON.parse r
    ary['list'].each do |l|
      res[l['id'].to_i] = l
    end
    res
  end
  
  def each_op
    Dir.glob File.expand_path(@dirname) + '/*' do |fn|
      ext = File.extname(fn).downcase
      bn = File.basename(fn, ".*")
      case ext
      when '.png', '.gif', '.jpeg'
        custom_attr_file = File.dirname(fn) + '/' + bn + '.json'
        attr = JSON.parse(File.open(custom_attr_file).read)
        yield fn, attr
      end
    end
  end
  
  def upload fn
    oid = nil
    b = Base64.encode64(File.open(fn, 'rb').read)
    u = @host + 'original_pictures'
    begin
      r = RestClient.post(u, 
        {:original_picture => {:file  => b}}.to_json, 
        :content_type => :json, :accept => :json, :email => @email, :token => @auth
      )
      o = JSON.parse r
      oid = o['id']
    rescue
      puts ' failed: ' + r.to_s
    end
    oid
  end
  
  def original_picture_license oid, lgid, lsid, en, attr
    rp = nil
    u = @host + '/original_picture_licenses'
    param = {
      :original_picture_id => oid, 
      :license_group_id => lgid, 
      :license_id => lsid, 
      :license_group_module_name => en, 
      :license_settings => attr, 
    }
    begin
      r = RestClient.post(u, 
        param.to_json,
        :content_type => :json, :accept => :json, :email => @email, :token => @auth
      )
      resp = JSON.parse r
      rp = resp['item']
    rescue
      puts ' failed: ' + u.to_s
      puts param
      puts r.to_s
    end
    rp
  end
  
  def publish oid, lgid, rp
    res = nil
    u = @host + 'resource_pictures'
    attr = {
      :original_picture_id => oid, 
      :original_picture_license_group => {
        :original_picture_id => oid, :license_group_id => lgid
      }, 
      :resource_picture => rp, 
    }
    begin
      r = RestClient.post(u, 
        attr.to_json,
        :content_type => :json, :accept => :json, :email => @email, :token => @auth
      )
      res = JSON.parse r
    rescue
      puts ' failed: ' + r.to_s
    end
    res
  end
  
  def publish_all
    each_op do |imgfile, attr|
      puts 'uploading: ' + imgfile
      oid = self.upload imgfile
      next unless oid
      puts 'success: ' + oid.to_s
      ls = @licenses[@lsname]
      unless ls
        puts 'not detect licenses: ' + @lsname.to_s
        next
      end
      lsid = ls['id']
      lgid = ls['license_group_id'].to_i
      lg = @license_groups[lgid]
      en = lg['module_name']
      confirm = self.original_picture_license oid, lgid, lsid, en, attr
      puts 'publishing: ' + @lsname.to_s
      r = self.publish oid, lgid, confirm
      puts 'success: ' + r['id'].to_s
    end
  end
  
end

u = Uploader.new host, dirname, lsname, email, auth
u.publish_all

