require 'picture_io'
class PictureIO
  Aws.config.update(
    region: Rails.application.secrets.aws_region, 
    credentials: Aws::Credentials.new(
      Rails.application.secrets.aws_access_key_id, 
      Rails.application.secrets.aws_secret_access_key
    )
  )
  class S3Picture
    @@client = Aws::S3::Client.new
    
    def initialize host_dir = 'pettanr', base_dir = ''
      @host_dir = host_dir
      @base_dir = base_dir
      s3 = Aws::S3::Resource.new
      @bucket = s3.bucket(@host_dir)
    end
    
    def self.client
      @@client
    end
    
    def bucket
      @bucket
    end
    
    def host_dir
      @host_dir
    end
    
    def host_dir=(d)
      @host_dir = d
    end
    
    def self.subdirs
      ['', 'v', 'h', 'vh', 'thumbnail']
    end
    
    def dir(subdir = nil)
      sd = if subdir.to_s.empty?
        ''
      else
        subdir.to_s + '/'
      end
      @base_dir + '/' + sd
    end
    
    def exist?(filename, subdir = nil)
      res = true
      begin
        res = self.bucket.object(dir(subdir) + filename).exists?
      rescue
        raise PictureIO::Error
      end
      res
    end
    
    def put(bindata, filename, subdir = nil)
      res = true
      begin
        @@client.put_object(bucket: self.host_dir, key: dir(subdir) + filename, body: bindata)
      rescue
        raise PictureIO::Error
      end
      res
    end
    
    def get(filename, subdir = nil)
      bindata = ''
      begin
        @@client.get_object(bucket: self.host_dir, key: dir(subdir) + filename) do |st|
          bindata += st if st
        end
      rescue
        raise PictureIO::Error
      end
      bindata
    end
    
    def delete(filename, subdir = nil)
      res = true
      begin
        @@client.delete_object(bucket: self.host_dir, key: dir(subdir) + filename)
      rescue
        raise PictureIO::Error
      end
      res
  end
    
  end
end
