# -*- encoding: utf-8 -*-
require 'spec_helper'
#貸手

describe ProvidersController do
  before do
    @admin = FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create :user_yas
    @author = @user.author    #ユーザ作成時に連動して作成される
  end
  
  describe '一覧表示に於いて' do
    before do
      @ps = FactoryGirl.create :provider_status
      @provider = FactoryGirl.create :provider, :provider_status_id => @ps.id
      Provider.stub(:list).and_return([@provider, @provider, @provider])
      Provider.stub(:available_list).and_return([@provider, @provider])
      sign_in @admin
    end
    context '事前チェックする' do
      it '与えられたpageがセットされている' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq Provider.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq Provider.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq Provider.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it '@hideが空になっている' do
        get :index
        assigns(:hide).should be_blank
      end
      it '貸手モデルに全一覧取得を問い合わせている' do
        Provider.should_receive(:list).exactly(1)
        get :index
      end
      it '@providersにリストを取得している' do
        get :index
        assigns(:providers).should have_at_least(3).items
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '貸手モデルにjson一覧出力オプションを問い合わせている' do
          Provider.should_receive(:list_json_opt).exactly(1)
          get :index, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいは貸手っぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("name").should be_true
          json.first.has_key?("caption").should be_true
          json.first.has_key?("url").should be_true
          json.first.has_key?("description").should be_true
        end
      end
    end
    context '除外フラグが除外のとき' do
      it '@hideが設定されている' do
        get :index, :hide => 1
        assigns(:hide).should_not be_blank
      end
      it '貸手モデルに待機中一覧取得を問い合わせている' do
        Provider.should_receive(:available_list).exactly(1)
        get :index, :hide => 1
      end
      it '@providersにリストを取得している' do
        get :index, :hide => 1
        assigns(:providers).should have_at_least(2).items
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.should redirect_to '/admins/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :index, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :index, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      @ps = FactoryGirl.create :provider_status
      @provider = FactoryGirl.create :provider, :provider_status_id => @ps.id
      Provider.stub(:show).and_return(@provider)
      sign_in @admin
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @provider.id
        response.should be_success
      end
      it '貸手モデルに単体取得を問い合わせている' do
        Provider.should_receive(:show).exactly(1)
        get :show
      end
      it '@providerにアレを取得している' do
        get :show, :id => @provider.id
        assigns(:provider).should eq(@provider)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @provider.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @provider.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '貸手モデルにjson単体出力オプションを問い合わせている' do
          Provider.should_receive(:show_json_opt).exactly(1)
          get :show, :id => @provider.id, :format => :json
        end
        it 'データがアレになっている' do
          get :show, :id => @provider.id, :format => :json
          json = JSON.parse response.body
          json.has_key?("name").should be_true
          json.has_key?("caption").should be_true
          json.has_key?("url").should be_true
          json.has_key?("description").should be_true
        end
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :show, :id => @provider.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :show, :id => @provider.id
          response.body.should redirect_to '/admins/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :show, :id => @provider.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :show, :id => @provider.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

end
