# -*- encoding: utf-8 -*-
require 'spec_helper'
#スクロール
describe ScrollsController do
  before do
    @admin =FactoryGirl.create :admin
    @user_yas = FactoryGirl.create :user_yas_with_owner
    @user_rom = FactoryGirl.create :user_rom
    @author_yas =  @user_yas.author
    
    @my_model_class =  Scroll   #操作モデル
  end

if Manifest.manifest.magic_numbers['run_mode'] == 1
  describe '一覧表示に於いて' do
    before do
      factory_contents :scroll_hello, @author_yas
      @query = { }
      @query_format_json = @query.merge(:format => :json)
    end
    
    context '正常系' do
      before do
        sign_in @user_yas
      end
      context '共通処理' do
        before do
          get :index, @query
        end
        it 'コントローラのマニフェストから操作モデルを導出している' do
          set_model? @my_model_class
        end
        it 'コントローラのマニフェストからリスト操作モデルを導出している' do
          set_list_model? @my_model_class
        end
        it 'リストグループのマニフェストからリストgetter(新着リスト)を導出している' do
          set_list? 'scroll', 'public'
        end
        it '@itemsにリストを取得している' do
          expect(assigns(:items).size).to be >= 3
        end
      end
      context 'html形式' do
        before do
          get :index, @query
        end
        it 'ステータス OKを返して、ファイラーを描画している' do
          return_ok?
          render_filer?
        end
        it '@filerにファイラーを取得している' do
          let_filer?
        end
      end
      context 'json形式' do
        before do
          get :index, @query_format_json
        end
        it 'ステータス OKを返して、jsonデータを返している' do
          return_ok?
          render_json?
        end
        it '@filerにファイラーをセットしていない' do
          expect(assigns(:filer)).to be nil
        end
      end
    end
    context '件数取得' do
      context 'html形式' do
        it '必要ないので実装していない' do
          sign_in @user_yas
          get :count, @query
          expect(response.status).to eq(406)
        end
      end
      context 'json形式' do
        it 'ステータス OKを返して、jsonデータを返している' do
          sign_in @user_yas
          get :count, @query_format_json
          return_ok?
          render_json?
        end
      end
      context 'ゲスト(サインインしていない)' do
        it '通常通り応答する' do
          get :count, @query_format_json
          return_ok?
          render_json?
        end
      end
    end
    context 'サインインの状態が例外的なとき' do
      it 'ユーザではなく管理者でも通常通り応答している' do
        sign_in @admin
        get :index, @query
        return_ok?
        get :index, @query_format_json
        render_json?
      end
      it '創作活動してないROM専の読者にも通常通り応答している' do
        sign_in @user_rom
        get :index, @query
        return_ok?
        get :index, @query_format_json
        render_json?
      end
      it 'ゲスト(サインインしていない)はサインインページに遷移している' do
        get :index, @query
        redirect? sign_in_url
        get :index, @query_format_json
        render_unauthorized?
      end
    end
  end
  describe '作家フィルタ一覧及びカウント' do
    before do
      factory_contents :scroll_hello, @author_yas
      @query = {:id => @author_yas.id}
      @query_format_json = @query.merge(:format => :json)
    end
    
    context '正常系' do
      before do
        sign_in @user_yas
      end
      context '共通処理' do
        before do
          get :by_author, @query
        end
        it 'コントローラのマニフェストから操作モデルを導出している' do
          set_model? @my_model_class
        end
        it 'リストグループのマニフェストから新着リストの取得処理を@listに取得している' do
          set_list? 'scroll', 'by_author'
        end
        it '@itemsにリストを取得している' do
          expect(assigns(:items).size).to be >= 3
        end
      end
      context 'html形式' do
        before do
          get :by_author, @query
        end
        it 'ステータス OKを返して、ファイラーを描画している' do
          return_ok?
          render_filer?
        end
        it '@filerにファイラーを取得している' do
          let_filer?
        end
      end
      context 'json形式' do
        before do
          get :by_author, @query_format_json
        end
        it 'ステータス OKを返して、jsonデータを返している' do
          return_ok?
          render_json?
        end
        it '@filerにファイラーをセットしていない' do
          expect(assigns(:filer)).to be nil
        end
      end
    end
    context '件数取得' do
      context 'html形式' do
        it '必要ないので実装していない' do
          sign_in @user_yas
          get :count_by_author, @query
          expect(response.status).to eq(406)
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          sign_in @user_yas
          get :count_by_author, @query_format_json
          return_ok?
          render_json?
        end
      end
      context 'ゲスト(サインインしていない)' do
        it '通常通り応答している' do
          get :count_by_author, @query_format_json
          return_ok?
          render_json?
        end
      end
    end
    context 'サインインの状態が例外的なとき' do
      it 'ユーザではなく管理者でも通常通り応答している' do
        sign_in @admin
        get :by_author, @query
        return_ok?
        get :by_author, @query_format_json
        render_json?
      end
      it '創作活動してないROM専の読者にも通常通り応答している' do
        sign_in @user_rom
        return_ok?
        get :by_author, @query_format_json
        render_json?
      end
      it 'ゲスト(サインインしていない)はサインインページに遷移している' do
        get :by_author, @query
        redirect? sign_in_url
        get :by_author, @query_format_json
        render_unauthorized?
      end
    end
  end
  
  describe 'コマフィルタ一覧及びカウント' do
    before do
      @panel = FactoryGirl.create :panel_hello_with_scroll_panels, author_id: @author_yas.id
      @query = {:id => @panel.id}
      @query_format_json = @query.merge(:format => :json)
    end
    
    context '正常系' do
      before do
        sign_in @user_yas
      end
      context '共通処理' do
        before do
          get :by_panel, @query
        end
        it 'コントローラのマニフェストから操作モデルを導出している' do
          set_model? @my_model_class
        end
        it 'リストグループのマニフェストから新着リストの取得処理を@listに取得している' do
          set_list? 'scroll', 'by_panel'
        end
        it '@itemsにリストを取得している' do
          expect(assigns(:items).size).to be >= 3
        end
      end
      context 'html形式' do
        before do
          get :by_panel, @query
        end
        it 'ステータス OKを返して、ファイラーを描画している' do
          return_ok?
          render_filer?
        end
        it '@filerにファイラーを取得している' do
          let_filer?
        end
      end
      context 'json形式' do
        before do
          get :by_panel, @query_format_json
        end
        it 'ステータス OKを返して、jsonデータを返している' do
          return_ok?
          render_json?
        end
        it '@filerにファイラーをセットしていない' do
          expect(assigns(:filer)).to be nil
        end
      end
    end
    context '件数取得' do
      context 'html形式' do
        it '必要ないので実装していない' do
          sign_in @user_yas
          get :count_by_panel, @query
          expect(response.status).to eq(406)
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          sign_in @user_yas
          get :count_by_panel, @query_format_json
          return_ok?
          render_json?
        end
      end
      context 'ゲスト(サインインしていない)' do
        it '通常通り応答している' do
          get :count_by_panel, @query_format_json
          return_ok?
          render_json?
        end
      end
    end
    context 'サインインの状態が例外的なとき' do
      it 'ユーザではなく管理者でも通常通り応答している' do
        sign_in @admin
        get :by_panel, @query
        return_ok?
        get :by_panel, @query_format_json
        render_json?
      end
      it '創作活動してないROM専の読者にも通常通り応答している' do
        sign_in @user_rom
        return_ok?
        get :by_panel, @query_format_json
        render_json?
      end
      it 'ゲスト(サインインしていない)はサインインページに遷移している' do
        get :by_panel, @query
        redirect? sign_in_url
        get :by_panel, @query_format_json
        render_unauthorized?
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      @item =  FactoryGirl.create :scroll_hello, author: @author_yas
      @item_id = @item.id
      @query = {:id => @item_id}
      @query_format_json = @query.merge(:format => :json)
      @query_format_prof = @query.merge(:format => :prof)
    end
    context '正常系' do
      before do
        sign_in @user_yas
      end
      context '正常系の共通処理' do
        before do
          get :show, @query
        end
        it 'コントローラのマニフェストから操作モデルを導出している' do
          set_model? @my_model_class
        end
        it '@itemにリクエストしたアイテムを取得している' do
          expect(assigns(:item)).to eq(@item)
        end
      end
      context 'html形式' do
        before do
          get :show, @query
        end
        it 'showテンプレートを描画している' do
          return_ok?
          expect(response).to render_template("show")
        end
        it 'プロファイラーは必要ないので@profilerに用意していない' do
          expect(assigns(:profiler)).to be nil
        end
      end
      context 'json形式' do
        before do
          get :show, @query_format_json
        end
        it 'jsonデータを返す' do
          return_ok?
          render_json?
        end
        it 'プロファイラーは必要ないので@profilerに用意していない' do
          expect(assigns(:profiler)).to be nil
        end
      end
      context 'prof形式' do
        before do
          get :show, @query_format_prof
        end
        it 'プロファイラーテンプレートを描画している' do
          return_ok?
          render_profiler?
        end
        it '@profilerにプロファイラーを取得している' do
          let_profiler?
        end
      end
    end
    context 'サインインの状態が例外的なとき' do
      it 'ユーザではなく管理者でも通常通り応答している' do
        sign_in @admin
        get :show, @query
        return_ok?
        get :show, @query_format_json
        render_json?
      end
      it '創作活動してないROM専の読者にも通常通り応答している' do
        sign_in @user_rom
        get :show, @query
        return_ok?
        get :show, @query_format_json
        render_json?
      end
      it 'ゲスト(サインインしていない)はサインインページに遷移している' do
        get :show, @query
        redirect? sign_in_url
        get :show, @query_format_json
        render_unauthorized?
      end
    end
  end
  
  describe 'プレイリストに於いて' do
    before do
      @item =  FactoryGirl.create :scroll_hello_with_scroll_panels, author: @author_yas
      @item_id = @item.id
      @query = {:id => @item_id}
      @query_format_json = @query.merge(:format => :json)
    end
    context '正常系' do
      before do
        sign_in @user_yas
      end
      context '共通処理' do
        before do
          get :play, @query
        end
        it 'コントローラのマニフェストから操作モデルを導出している' do
          set_model? @my_model_class
        end
        it '@itemにリクエストしたスクロールを取得している' do
          expect(assigns(:item)).to eq(@item)
        end
        it 'コントローラのマニフェストからリスト操作モデルを導出している' do
          # リクエストの対象はスクロールであるが、返されるのはスクコマのリストである。
          # くれぐれも間違えないように。 
          set_list_model? ScrollPanel
        end
        it 'リストグループのマニフェストからリストgetter(プレイリスト)を導出している' do
          set_list? 'scroll_panel', 'play'
        end
        it '@itemsにプレイリストを取得している' do
          set_play_list? ScrollPanel
        end
      end
      context 'html形式' do
        before do
          get :play, @query
        end
        it 'playテンプレートを描画している' do
          return_ok?
          expect(response).to render_template("play")
        end
      end
      context 'json形式' do
        before do
          get :play, @query_format_json
        end
        it 'jsonデータを返す' do
          return_ok?
          render_json?
        end
      end
    end
    context 'サインインの状態が例外的なとき' do
      it 'ユーザではなく管理者でも通常通り応答している' do
        sign_in @admin
        get :play, @query
        return_ok?
        get :play, @query_format_json
        render_json?
      end
      it '創作活動してないROM専の読者にも通常通り応答している' do
        sign_in @user_rom
        get :play, @query
        return_ok?
        get :play, @query_format_json
        render_json?
      end
      it 'ゲスト(サインインしていない)はサインインページに遷移している' do
        get :play, @query
        redirect? sign_in_url
        get :play, @query_format_json
        render_unauthorized?
      end
    end
  end
  
  describe '新規作成フォーム表示に於いて' do
    before do
      @query = { }
      @query_format_json = @query.merge(:format => :json)
    end
    context '正常系' do
      before do
        sign_in @user_yas
      end
      context '共通処理' do
        before do
          get :new, @query
        end
        it 'コントローラのマニフェストから操作モデルを導出している' do
          set_model? @my_model_class
        end
        it '@itemに新規データを用意している' do
          @item = assigns(:item)
          expect(@item.is_a?(@my_model_class)).to be true
          expect(@item.new_record?).to be true
        end
        it 'ブーストしている' do
          boost? []
        end
      end
      context '問い合わせ' do
        it 'スクロールモデルにデフォルト値補充を依頼している' do
          allow_any_instance_of(@my_model_class).to receive(:supply_default)
          expect_any_instance_of(@my_model_class).to receive(:supply_default)
          get :new, @query
        end
      end
      context 'html形式' do
        before do
          get :new, @query
        end
        it 'formテンプレートを描画している' do
          return_ok?
          render_form?
        end
        it '@formに入力フォーム(Bucket)を取得している' do
          set_bucket? assigns(:item)
        end
      end
      context 'json形式' do
        before do
          get :new, @query_format_json
        end
        it 'jsonデータを返す' do
          return_ok?
          render_json?
        end
        it '入力フォームは必要ないので@formに用意していない' do
          expect(assigns(:form)).to be nil
        end
      end
    end
    context 'サインインの状態が例外的なとき' do
      before do
      end
      it '創作活動してないROM専の読者は作家登録ページへ遷移している, 例外を発生させている' do
        sign_in @user_rom
        get :new, @query
        redirect? new_author_path
        announce_regist_author_for_json :get, @query_format_json, :new
      end
      it 'ゲスト(サインインしていない)はサインインページに遷移している' do
        get :new, @query
        redirect? sign_in_url
        get :new, @query_format_json
        render_unauthorized?
      end
      it 'ユーザではなく管理者でもサインインページに遷移している' do
        sign_in @admin
        get :new, @query
        redirect? sign_in_url
        get :new, @query_format_json
        render_unauthorized?
      end
    end
  end

  describe '新規作成に於いて' do
    before do
      @attributes = FactoryGirl.attributes_for(:scroll_hello, author_id: @author_yas.id)
      @attr = {:scroll => @attributes}
      @attr_format_json = @attr.merge(:format => :json)
    end
    context '正常系' do
      before do
        sign_in @user_yas
      end
      context '共通処理' do
        before do
          allow_save @my_model_class
          post :create, @attr
        end
        it 'コントローラのマニフェストから操作モデルを導出している' do
          set_model? @my_model_class
        end
        it '@itemに新規アイテムを用意している' do
          @item = assigns(:item)
          expect(@item.is_a?(@my_model_class)).to be true
        end
      end
      context 'html形式' do
        before do
          post :create, @attr
        end
        it '作成されたアイテムの表示ページへ遷移している' do
          redirect? @my_model_class.last
        end
      end
      context 'json形式' do
        before do
          post :create, @attr_format_json
        end
        it 'ステータスコード200 OKを返して, 作成されたアイテムをjsonデータで返す' do
          return_ok?
          render_json?
        end
      end
      context 'db' do
        it '作成されている' do
          expect {
            post :create, @attr
          }.to change {@my_model_class.count}.by(1)
        end
      end
    end
    context '検証、保存に失敗したとき' do
      before do
        sign_in @user_yas
        reject_save @my_model_class
      end
      context 'html形式' do
        before do
          post :create, @attr
        end
        it 'ステータスコード200 OKを返して, formテンプレートを描画している' do
          return_ok?
          render_form?
        end
        it '@formに入力フォーム(Bucket)を取得している' do
          set_bucket? assigns(:item)
        end
      end
      context 'json形式' do
        before do
          post :create, @attr_format_json
        end
        it '処理不能を返している' do
          render_unprocessable?
        end
      end
    end
    context 'サインインの状態が例外的なとき' do
      before do
        allow_save @my_model_class
      end
      it '創作活動してないROM専の読者は作家登録ページへ遷移している, 例外を発生させている' do
        sign_in @user_rom
        post :create, @attr
        redirect? new_author_path
        announce_regist_author_for_json :post, @attr_format_json, :create
      end
      it 'ゲスト(サインインしていない)はサインインページに遷移している' do
        post :create, @attr
        redirect? sign_in_url
        post :create, @attr_format_json
        render_unauthorized?
      end
      it 'ユーザではなく管理者でもサインインページに遷移している' do
        sign_in @admin
        post :create, @attr
        redirect? sign_in_url
        post :create, @attr_format_json
        render_unauthorized?
      end
    end
  end

  describe '編集フォーム表示に於いて' do
    before do
      @item =  FactoryGirl.create :scroll_hello_with_scroll_panels, author: @author_yas
      @item_id = @item.id
      @query = {:id => @item_id}
      @query_format_json = @query.merge(:format => :json)
    end
    context '正常系' do
      before do
        sign_in @user_yas
      end
      context '共通処理' do
        before do
          get :edit, @query
        end
        it 'コントローラのマニフェストから操作モデルを導出している' do
          set_model? @my_model_class
        end
        it '@itemにリクエストしたアイテムを取得している' do
          expect(assigns(:item).is_a?(@my_model_class)).to be true
          expect(assigns(:item)).to eq(@item)
        end
        it 'ブーストしている' do
          boost? []
        end
      end
      context 'html形式' do
        before do
          get :edit, @query
        end
        it 'formテンプレートを描画している' do
          return_ok?
          render_form?
        end
        it '@formに入力フォーム(Bucket)を取得している' do
          set_bucket? assigns(:item)
        end
      end
      context 'json形式' do
        before do
          get :edit, @query_format_json
        end
        it 'jsonデータを返す' do
          return_ok?
          render_json?
        end
        it '入力フォームは必要ないので@formに用意していない' do
          expect(assigns(:form)).to be nil
        end
      end
    end
    context 'サインインの状態が例外的なとき' do
      before do
        allow_save @my_model_class
      end
      it '創作活動してないROM専の読者は作家登録ページへ遷移している, 例外を発生させている' do
        sign_in @user_rom
        get :edit, @query
        redirect? new_author_path
        announce_regist_author_for_json :get, @query_format_json, :edit
      end
      it 'ゲスト(サインインしていない)はサインインページに遷移している' do
        post :edit, @query
        redirect? sign_in_url
        post :edit, @query_format_json
        render_unauthorized?
      end
      it 'ユーザではなく管理者でもサインインページに遷移している' do
        sign_in @admin
        post :edit, @query
        redirect? sign_in_url
        post :edit, @query_format_json
        render_unauthorized?
      end
    end
  end

  describe '更新に於いて' do
    before do
      @item =  FactoryGirl.create :scroll_hello_with_scroll_panels, author: @author_yas
      @item_id = @item.id
      @attributes = FactoryGirl.attributes_for(:scroll_hidden, author_id: @author_yas.id)
      @attr = {:id => @item_id, :scroll => @attributes}
      @attr_format_json = @attr.merge(:format => :json)
    end
    context '正常系' do
      before do
        sign_in @user_yas
      end
      context '共通処理' do
        before do
          allow_save @my_model_class
          put :update, @attr
        end
        it 'コントローラのマニフェストから操作モデルを導出している' do
          set_model? @my_model_class
        end
        it '@itemにリクエストしたアイテムを取得している' do
          expect(assigns(:item).is_a?(@my_model_class)).to be true
          expect(assigns(:item)).to eq(@item)
        end
      end
      context 'html形式' do
        before do
          allow_save @my_model_class
          put :update, @attr
        end
        it 'リクエストしたアイテムの表示ページへ遷移している' do
          redirect? @item
        end
      end
      context 'json形式' do
        before do
          allow_save @my_model_class
          put :update, @attr_format_json
        end
        it 'ステータスコード200 OKを返して, ページ本体は特に返さない' do
          return_ok?
          render_complated?
        end
      end
      context 'db' do
        it '@itemの件数に変わりはない' do
          expect {
            put :update, @attr
          }.to change {@my_model_class.count}.by(0)
        end
        it '@itemを更新している' do
          put :update, @attr
          # dbから取得しなおす
          updated_item = @my_model_class.find @item_id
          # 変化を確認できるカラムはモデルごとに違うので注意
          expect(updated_item.visible).to eq(0)
        end
      end
    end
    context '検証、保存に失敗したとき' do
      before do
        sign_in @user_yas
        reject_save @my_model_class
      end
      context 'html形式' do
        before do
          put :update, @attr
        end
        it 'ステータスコード200 OKを返して, formテンプレートを描画している' do
          return_ok?
          render_form?
        end
        it '@formに入力フォーム(Bucket)を取得している' do
          set_bucket? assigns(:item)
        end
      end
      context 'json形式' do
        before do
          put :update, @attr_format_json
        end
        it '処理不能を返している' do
          render_unprocessable?
        end
        it '入力フォームは必要ないので@formに用意していない' do
          expect(assigns(:form)).to be nil
        end
      end
    end
    context 'サインインの状態が例外的なとき' do
      before do
        allow_save @my_model_class
      end
      it '創作活動してないROM専の読者は作家登録ページへ遷移している, 例外を発生させている' do
        sign_in @user_rom
        put :update, @attr
        redirect? new_author_path
        announce_regist_author_for_json :put, @attr_format_json, :update
      end
      it 'ゲスト(サインインしていない)はサインインページに遷移している' do
        put :update, @attr
        redirect? sign_in_url
        put :update, @attr_format_json
        render_unauthorized?
      end
      it 'ユーザではなく管理者でもサインインページに遷移している' do
        sign_in @admin
        put :update, @attr
        redirect? sign_in_url
        put :update, @attr_format_json
        render_unauthorized?
      end
    end
  end

  describe '削除に於いて' do
    before do
      @item =  FactoryGirl.create :scroll_hello_with_scroll_panels, author: @author_yas
      @item_id = @item.id
      @attr = {:id => @item_id}
      @attr_format_json = @attr.merge(:format => :json)
    end
    context '正常系' do
      before do
        sign_in @user_yas
      end
      context '共通処理' do
        before do
          allow_destroy @my_model_class
          delete :destroy, @attr
        end
        it 'コントローラのマニフェストから操作モデルを導出している' do
          set_model? @my_model_class
        end
        it '@itemにリクエストしたアイテムを取得している' do
          expect(assigns(:item).is_a?(@my_model_class)).to be true
          expect(assigns(:item)).to eq(@item)
        end
      end
      context 'html形式' do
        before do
          allow_destroy @my_model_class
          delete :destroy, @attr
        end
        it 'マイスクロールの一覧ページへ遷移している' do
          redirect? '/home/scrolls'
        end
      end
      context 'json形式' do
        before do
          allow_destroy @my_model_class
          delete :destroy, @attr_format_json
        end
        it 'ステータスコード200 OKを返して, ページ本体は特に返さない' do
          expect(response).to be_success
          render_complated?
        end
      end
      context 'db' do
        it '削除されている' do
          expect {
            delete :destroy, @attr
          }.to change {@my_model_class.count}.by(-1)
        end
        it 'ぶら下がるリーフも削除されている' do
          expect {
            delete :destroy, @attr
          }.to change {ScrollPanel.count}.from(4).to(0)
        end
      end
    end
    context '検証、保存に失敗したとき' do
      before do
        sign_in @user_yas
        reject_destroy @my_model_class
      end
      context 'html形式' do
        before do
          delete :destroy, @attr
        end
        it 'リクエストしたアイテムの詳細ページへ遷移している' do
          redirect? @item
        end
      end
      context 'json形式' do
        before do
          delete :destroy, @attr_format_json
        end
        it '処理不能を返している' do
          render_unprocessable?
        end
      end
    end
    context 'サインインの状態が例外的なとき' do
      before do
        allow_destroy @my_model_class
      end
      it '創作活動してないROM専の読者は作家登録ページへ遷移している, 例外を発生させている' do
        sign_in @user_rom
        delete :destroy, @attr
        redirect? new_author_path
        announce_regist_author_for_json :delete, @attr_format_json, :destroy
      end
      it 'ゲスト(サインインしていない)はサインインページに遷移している' do
        delete :destroy, @attr
        redirect? sign_in_url
        delete :destroy, @attr_format_json
        render_unauthorized?
      end
      it 'ユーザではなく管理者でもサインインページに遷移している' do
        sign_in @admin
        delete :destroy, @attr
        redirect? sign_in_url
        delete :destroy, @attr_format_json
        render_unauthorized?
      end
    end
  end

else
  describe '一覧表示に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @user.author.id
      Scroll.stub(:list).and_return([@scroll, @scroll, @scroll])
      sign_in @user
    end
    context '正常系の共通処理' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @user.author.id, :title => 'normal'
      Scroll.stub(:show).and_return(@scroll)
      sign_in @user
    end
    context '正常系の共通処理' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @scroll.id
        response.should be_success
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @scroll.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @scroll.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @scroll.id
        response.should be_success
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @scroll.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @scroll.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
  end
  describe 'スクロール数取得に於いて' do
    before do
      Scroll.should_receive(:visible_count).and_return(3)
      sign_out @user
    end
    context '正常系の共通処理' do
      it 'ステータスコード200 OKを返す' do
        get :count, :format => :json
        response.should be_success 
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :count, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
  end

  describe '新規作成フォーム表示に於いて' do
    before do
      sign_in @user
    end
    context '正常系の共通処理' do
      it 'ステータスコード200 OKを返す' do
        get :new
        response.should be_success 
      end
      context 'html形式' do
        it 'newテンプレートを描画する' do
          get :new
          response.should render_template("new")
        end
      end
      context 'js形式' do
        it 'new.jsテンプレートを描画する' do
          get :new, :format => :js
          response.should render_template("new")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :new, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :new
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :new
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'js形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :new, :format => :js
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :new, :format => :js
          response.message.should match(/Unauthorized/)
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :new, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :new, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '新規作成に於いて' do
    before do
      sign_in @user
      @attr = FactoryGirl.attributes_for(:scroll, :author_id => @author.id, :title => 'normal')
    end
    context '正常系の共通処理' do
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          Scroll.any_instance.stub(:save).and_return(true)
          post :create, :scroll => @attr
          response.status.should eq 302
        end
        it '作成されたスクロールの表示ページへ遷移する' do
#          Scroll.any_instance.stub(:save).and_return(true)
          post :create, :scroll => @attr
          response.should redirect_to(Scroll.last)
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
#          Scroll.any_instance.stub(:save).and_return(true)
          post :create, :scroll => @attr, :format => :json
          response.should be_success 
        end
        it '作成されたスクロールをjsonデータで返す' do
          post :create, :scroll => @attr, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :create, :scroll => @attr
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          post :create, :scroll => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          post :create, :scroll => @attr, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          post :create, :scroll => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '編集フォーム表示に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @user.author.id
      sign_in @user
      Scroll.stub(:edit).and_return(@scroll)
    end
    context '正常系の共通処理' do
      it 'ステータスコード200 OKを返す' do
        get :edit, :id => @scroll.id
        response.should be_success 
      end
      context 'html形式' do
        it 'editテンプレートを描画する' do
          get :edit, :id => @scroll.id
          response.should render_template("edit")
        end
      end
      context 'js形式' do
        it 'edit.jsテンプレートを描画する' do
          get :edit, :id => @scroll.id, :format => :js
          response.should render_template("edit")
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :edit, :id => @scroll.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :edit, :id => @scroll.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'js形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :edit, :id => @scroll.id, :format => :js
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :edit, :id => @scroll.id, :format => :js
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '更新に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author => @author
      @attr = FactoryGirl.attributes_for(:scroll, :author_id => @author.id, :title => 'updated title', :visible => 0)
      sign_in @user
    end
    context '正常系の共通処理' do
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          Scroll.any_instance.stub(:save).with(any_args()).and_return(true)
          put :update, :id => @scroll.id, :scroll => @attr
          response.status.should eq 302
        end
        it '更新されたスクロールの表示ページへ遷移する' do
          put :update, :id => @scroll.id, :scroll => @attr
          response.should redirect_to(@scroll)
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          Scroll.any_instance.stub(:save).with(any_args()).and_return(true)
          put :update, :id => @scroll.id, :scroll => @attr, :format => :json
          response.should be_success 
        end
        it 'ページ本体は特に返さない' do
          Scroll.any_instance.stub(:save).with(any_args()).and_return(true)
          put :update, :id => @scroll.id, :scroll => @attr, :format => :json
          response.body.should match /./
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード302 Foundを返す' do
        put :update, :id => @scroll.id, :scroll => @attr
        response.status.should eq 302
      end
      context 'html形式' do
        it 'サインインページへ遷移する' do
          put :update, :id => @scroll.id, :scroll => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
    end
  end

  describe '削除に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author => @author
      sign_in @user
    end
    context '正常系の共通処理' do
      context 'html形式' do
        before do
          Scroll.any_instance.stub(:destroy_with_scroll_panel).with(any_args()).and_return(true)
        end
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @scroll.id
          response.status.should eq 302
        end
        it 'マイスクロールの一覧ページへ遷移する' do
          delete :destroy, :id => @scroll.id
          response.should redirect_to('/home/scroll')
        end
      end
      context 'json形式' do
        before do
          Scroll.any_instance.stub(:destroy_with_scroll_panel).with(any_args()).and_return(true)
        end
        it 'ステータスコード200 OKを返す' do
          delete :destroy, :id => @scroll.id, :format => :json
          response.should be_success 
        end
        it 'ページ本体は特に返さない' do
          delete :destroy, :id => @scroll.id, :format => :json
          response.body.should match /./
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード302 Foundを返す' do
        delete :destroy, :id => @scroll.id
        response.status.should eq 302
      end
      context 'html形式' do
        it 'サインインページへ遷移する' do
          delete :destroy, :id => @scroll.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it '応答メッセージにUnauthorizedを返す' do
          delete :destroy, :id => @scroll.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

end

end
